<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Teknisa\CRM\Service\EnderecoParceiroService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class EnderecoParceiroController extends TeknisaCrud {

    protected $crudService;
    protected $entityManager;
    protected $connection;
    protected $dataSourceName = 'enderecoparc_endereco';
    protected $enderecoParceiroService;
    protected $parameterBagColumns = array("P_NRPARCNEGOCIO", "P_NRORG", "P_NRORGPADRAO");

    public function __construct($crudService, ParameterBag $parameterBag, $entityManager, EnderecoParceiroService $enderecoParceiroService) {
        $this->crudService = $crudService;
        $this->connection  = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->enderecoParceiroService = $enderecoParceiroService;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function findEndereco(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $findEndereco = "SELECT NRENDERECOPARC FROM ENDERECOPARC
                             WHERE NRPARCNEGOCIO = :P_NRPARCNEGOCIOG
                               AND NRORG = :P_NRORG";
            
            $params = array (
                "P_NRPARCNEGOCIOG"      => $row['NRPARCNEGOCIO'],
                "P_NRORG"             => $row['NRORG']
            );
            
            $std = $this->connection->prepare($findEndereco);
            $std->execute($params);
            $nrenderecoparc = $std->fetchAll();
            $response->addDataset(new DataSet('columnsToReturn', $nrenderecoparc));
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $nrenderecoparc = $this->enderecoParceiroService->saveEnderecoParceiro($row);
            if ($row['__is_new']) {
                $response->addDataset(new DataSet('columnsToReturn', array("NRENDERECOPARC" => $nrenderecoparc)));
            }
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->enderecoParceiroService->removeEnderecoParceiro($row);
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

    }
}