<?php

namespace Teknisa\CRM\Service;

use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Session\Session;
use Teknisa\Model\Util\NovoCodigo;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;

class IntegracaoSmService {
    
    const CLIENT_ID      = 'cd7e85178153by60953518945f9hakt19a50a4';
    const CLIENT_SECRET  = 'lib5449996c21f780fe30badd6521931a5sac9';
    const URL_INTEGRACAO = 'https://sm.teknisa.com/backend/index.php';
    
    protected $environment;
    protected $session;
    
    public function __construct(Environment $environment, Session $session, $oportEvolController, $oportController) {
        $this->environment              = $environment;
        $this->session                  = $session;
        $this->oportEvolController      = $oportEvolController;
        $this->oportController          = $oportController;
    }
    
    /**
     * Executa uma requisição curl.
     * 
     * @param string  $token
     * @param string  $rota
     * @param mixed[] $dados
     * 
     * @return mixed[]
     */
    private function executarCurl($token, $rota, $dados) {
        set_time_limit(180);
        $header = array();
        $header[] = "OAuth-Token: " . $token;
        $header[] = "OAuth-SecretId: " . self::CLIENT_SECRET;
        $header[] = "Content-Type: application/json";
        $header[] = "Content-Length: " . strlen(json_encode($dados));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, self::URL_INTEGRACAO . $rota);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($dados));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        
        $resultado = curl_exec($ch);
        $resultado = json_decode($resultado, true);
        $error = curl_error($ch);
        curl_close($ch);
        
        if (!empty($resultado["error"])) {
            throw new \Exception($resultado["error"]);
        }
        else if (!empty($error)) {
            throw new \Exception($error);
        }
        else {
            return $resultado["dataset"];
        }
    }
    
    /**
     * Retorna o token para a comunicação curl.
     * 
     * @return string
     */
    private function getToken() {
        $token = $this->session->get("lib_token");
        if ($token)
            return $token;
        else {
            $param = array(
                "CLIENT_ID"     => self::CLIENT_ID,
                "CLIENT_SECRET" => self::CLIENT_SECRET
            );
            $token = $this->executarCurl(null, "/lib_requestToken", $param);
            
            $token = $token["token"][0];
            $this->session->set("lib_token", $token);
            return $token;
        }
    }
    
    /**
     * Envia o dados da proposta para o SM.
     * 
     * @param array   $dados
     * @param string  $rota
     */
    public function geraProposta($dados, $rota) {
        try {
            
            $dados = array(
                "requestType" => "DataSet",
                "dataset"     => array($dados)
            );
            
            $token = $this->getToken();
            $retorno = $this->executarCurl($token, $rota, $dados);
            return $retorno;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraEvolucaoOportunidade($row, $response, $entityManager = null){
        
        $returnMessage  = '';  
        $evolucao       = $row['evolucao'];
        $oportunidade   = $row['oportunidade'];
        $proxima_tarefa = $row['proxima_tarefa'];
        
        $proxima_tarefa['nrseqfilaconta'] = null;
        if(!empty($proxima_tarefa['data'])){
            $rowTarefa = array(
                'DSFILACONTATO'   => $proxima_tarefa['assunto'],
                'DTPLANFILACONTA' => $proxima_tarefa['data'],
                'CDOPERREAL'      => $proxima_tarefa['operador'],
                'CDOPEREVOL'      => $evolucao['operador'],
                'NRPARCNEGOCIO'   => $oportunidade['nrparcnegocio'],
                'NRSEQCONTATO'    => $oportunidade['nrseqcontato'],
                'NRPRIORFILACONT' => '',
                'NRORG'           => $row['requisitante']['nrorg']    
            );
            $request = new Row($rowTarefa, '', '', '');
            $nrseqfilaconta = $this->oportEvolController->inserirTarefa($request, $response, $entityManager);
            $proxima_tarefa['nrseqfilaconta'] = $nrseqfilaconta;
            $returnMessage .= 'Tarefa Criada; ';
        }
        
        $rowOport = array(
            'NRSEQOPORT'    => $oportunidade['nrseqoport'], 
            'NRSEQFASENEGO' => $oportunidade['fase'],
            'NRPARCRESP'    => $oportunidade['novo_operador']['parceiro'],
            'VROPORT'       => $oportunidade['fechamento']['valor'],
            'VRPREVOPORT'   => $oportunidade['valor_previsto'], 
            'NRTERMOPORTUNI'=> $oportunidade['termometro'],
            'NRORG'         => $row['requisitante']['nrorg']
        );
        $request = new Row($rowOport, '', '', '');
        $this->oportEvolController->atualizarOportunidade($request, $response, $entityManager);
        $returnMessage .= 'Oportunidade Atualizada; ';
        
        if(!empty($oportunidade['fechamento']['data'])){
            $rowFinishOport = array(
                'NRSEQOPORT'    => $oportunidade['nrseqoport'], 
                'DTREALFECHA'   => $oportunidade['fechamento']['data'],
                'NRSEQRESULEVOL'=> $oportunidade['fechamento']['resultado'],
                'NRCONCORRENTE' => $oportunidade['fechamento']['nrparcnegocio_concorrente'],
                'NRORG'         => $row['requisitante']['nrorg']
            );
            $request = new Row($rowFinishOport, '', '', '');
            $this->oportController->fecharOportunidade($request, $response, $entityManager);
            $returnMessage .= 'Oportunidade Finalizada; ';
        }
        
        $rowEvolucao = array(
            '__is_new'          => true,
            'IDGERAFATURA'      => 'NAO',
            'IDTPEVOLCONT'      => 'P',
            'IDANEXOEVOLCONT'   => 'N',
            'NRCONTATO'         => $oportunidade['nrseqcontato'],
            'DTHREVOLCONT'      => $evolucao['data'], 
            'CDTIPOCONT'        => $evolucao['contato'], 
            'IDSENEVOCONT'      => $evolucao['sentido'], 
            'CDOPEREVOL'        => $evolucao['operador'],
            'CDMOTIEVOCON'      => $evolucao['motivo'], 
            'NRORG'             => $this->environment->getNrorgTrab(),
            'TXEVOLCONT'        => $evolucao['descricao'],
            'IDATIVO'           => 'S', 
            'NRSEQRESULEVOL'    => $evolucao['resultado'], 
            'NRSEQCONTATO'      => $oportunidade['nrseqcontato'], 
            'NRSEQFILACONTA'    => $proxima_tarefa['nrseqfilaconta'], 
            'NRSEQFASENEGO'     => $oportunidade['fase'], 
            'NRTERMOPORTUNI'    => $oportunidade['termometro'],
            'NRSEQINTERLOCUTOR' => $evolucao['interlocutor'],
            'NRORG'             => $row['requisitante']['nrorg']
        );
        $datasource = new DataSet('EVOLUCAO', array($rowEvolucao));
        $request = new Request\DataSet($datasource, '', '', '');
        $this->oportEvolController->saveIntegracao($request, $response, $entityManager);
        $returnMessage .= 'Evolução Criada; ';
        return $returnMessage;
    }
}

?>